#include "NamedCounterFactory.h"

using std::string;

NamedCounterFactoryImpl::NamedCounterFactoryImpl()
{
}

NamedCounterFactoryImpl::~NamedCounterFactoryImpl()
{
   CounterContainer::iterator it = m_counters.begin();
   
   while (it != m_counters.end())
   {
      delete (*it).second;
      ++it;
   }
}

NamedCounter_ptr NamedCounterFactoryImpl::GetNamedCounter(
   const char* name)
{
   CounterContainer::const_iterator it = m_counters.find(string(name));

   if (it == m_counters.end())
   {
      // not found

      throw NamedCounterFactory::NoCounterExists(name);
   }

   return NamedCounter::_duplicate((*it).second->_this());
}

void NamedCounterFactoryImpl::AddNamedCounter(
   const char* name, 
   CORBA::Long initialValue)
{
   CounterContainer::const_iterator it = m_counters.find(string(name));

   if (it != m_counters.end())
   {
      // found

      throw NamedCounterFactory::CounterExists(name);
   }

   NamedCounterImpl *pCounter = new NamedCounterImpl(name, initialValue);

   m_counters.insert(CounterContainer::value_type(name, pCounter));
}

void NamedCounterFactoryImpl::RemoveNamedCounter(
   const char* name)
{
   CounterContainer::iterator it = m_counters.find(string(name));

   if (it == m_counters.end())
   {
      // not found

      throw NamedCounterFactory::NoCounterExists(name);
   }

   // this erases the map entry between the name of the counter and
   // the pointer to the counter object, unfortunately we shouldn't delete
   // the counter object itself because:
   // 1) the CORBA object that this servant object is providing an 
   // implementation for is still active and...
   // 2) We could leave clients with references to a CORBA object that 
   // no longer has a servant object - and worse still, the POA doesn't 
   // know that there's no longer a servant object!

   delete (*it).second;

   // Commenting out the line above removes the two problems above and
   // instead adds a memory leak to the server.

   m_counters.erase(it);   
}
  

NamedCounterFactory::counterSeq *NamedCounterFactoryImpl::GetAll()
{
   const size_t numCounters = m_counters.size();


   NamedCounterFactory::counterSeq_var seq = new NamedCounterFactory::counterSeq(numCounters);

	seq->length(numCounters);

   size_t i = 0;

   CounterContainer::const_iterator it = m_counters.begin();
   
   while (it != m_counters.end())
   {
      seq[i] = NamedCounter::_duplicate((*it).second->_this());
      
      ++it;
      ++i;
   }

   return seq._retn();
}
